MODULE bdydyn
   !!=================================================================================
   !!                       ***  MODULE  bdydyn  ***
   !! Ocean dynamics:   Flow relaxation scheme of velocities on unstruc. open boundary
   !!=================================================================================

   !!---------------------------------------------------------------------------------
   !!   bdy_dyn_frs    : relaxation of velocities on unstructured open boundary
   !!   bdy_dyn_fla    : Flather condition for barotropic solution
   !!---------------------------------------------------------------------------------
#if defined key_bdy || defined key_bdy_tides
   !!---------------------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers 
   USE dom_oce         ! ocean space and time domain
   USE bdy_oce         ! ocean open boundary conditions
   USE dynspg_oce      ! for barotropic variables
   USE phycst          ! physical constants
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)
   USE bdytides        ! for tidal harmonic forcing at boundary
   USE in_out_manager

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC bdy_dyn_frs  ! routine called in dynspg_flt (free surface case ONLY)
#if defined key_dynspg_exp || defined key_dynspg_ts
   PUBLIC bdy_dyn_fla  ! routine called in dynspg_flt (free surface case ONLY)
#endif

   !!---------------------------------------------------------------------------------

CONTAINS

   SUBROUTINE bdy_dyn_frs ( kt )
      !!------------------------------------------------------------------------------
      !!                      SUBROUTINE bdy_dyn_frs
      !!                     ***********************
      !! ** Purpose : - Apply the Flow Relaxation Scheme for dynamic in the  
      !!                case of unstructured open boundaries.
      !!
      !! References :- Engedahl H., 1995: Use of the flow relaxation scheme in 
      !!               a three-dimensional baroclinic ocean model with realistic
      !!               topography. Tellus, 365-382.
      !!
      !! History :
      !!   9.0  !  05-02 (J. Chanut, A. Sellar) Original
      !!        !  07-07 (D. Storkey) Move Flather implementation to separate routine.
      !!------------------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt    ! Main time step counter

      !! * Local declarations
      INTEGER ::   jb, jk, jgrd        ! dummy loop indices
      INTEGER ::   ii, ij              ! 2D addresses
      REAL(wp) :: zwgt                 ! boundary weight

      !!------------------------------------------------------------------------------
      !!  OPA 9.0, LODYC-IPSL (2003)
      !!------------------------------------------------------------------------------

      ! ---------------------------!
      ! FRS on the total velocity :!
      ! ---------------------------!  
  
      jgrd=2 !: Relaxation of zonal velocity
      DO jb = 1, nblen(jgrd)
        DO jk = 1, jpkm1
          ii = nbi(jb,jgrd)
          ij = nbj(jb,jgrd)
          zwgt = nbw(jb,jgrd)

          ua(ii,ij,jk) = ( ua(ii,ij,jk)*(1.-zwgt) + ubdy(jb,jk)*zwgt ) &
                                                        * umask(ii,ij,jk)
        END DO
      END DO

      jgrd=3 !: Relaxation of meridional velocity
      DO jb = 1, nblen(jgrd)
        DO jk = 1, jpkm1
          ii = nbi(jb,jgrd)
          ij = nbj(jb,jgrd)
          zwgt = nbw(jb,jgrd)

          va(ii,ij,jk) = ( va(ii,ij,jk)*(1.-zwgt) + vbdy(jb,jk)*zwgt ) &
                                                        * vmask(ii,ij,jk)
        END DO
      END DO 

      CALL lbc_lnk( ua, 'U', 1. ) ! Boundary points should be updated
      CALL lbc_lnk( va, 'V', 1. ) !

   END SUBROUTINE bdy_dyn_frs

#if defined key_dynspg_exp || defined key_dynspg_ts
!! Option to use Flather with dynspg_flt not coded yet...
   SUBROUTINE bdy_dyn_fla
      !!------------------------------------------------------------------------------
      !!                      SUBROUTINE bdy_dyn_fla
      !!                     ***********************
      !!              - Apply Flather boundary conditions on normal barotropic velocities 
      !!                (ln_bdy_fla=.true.)
      !!
      !! ** WARNINGS about FLATHER implementation:
      !!1. According to Palma and Matano, 1998 "after ssh" is used. 
      !!   In ROMS and POM implementations, it is "now ssh". In the current 
      !!   implementation (tested only in the EEL-R5 conf.), both cases were unstable. 
      !!   So I use "before ssh" in the following.
      !!
      !!2. We assume that the normal ssh gradient at the bdy is zero. As a matter of 
      !!   fact, the model ssh just inside the dynamical boundary is used (the outside  
      !!   ssh in the code is not updated).
      !!
      !!             - Flather, R. A., 1976: A tidal model of the northwest European
      !!               continental shelf. Mem. Soc. R. Sci. Liege, Ser. 6,10, 141-164.     
      !! History :
      !!   9.0  !  05-02 (J. Chanut, A. Sellar) Original
      !!        !  07-07 (D. Storkey) Flather algorithm in separate routine.
      !!------------------------------------------------------------------------------
      !! * Local declarations
      INTEGER ::   jb, jk, jgrd, ji, jj, jim1, jip1, jjm1, jjp1   ! dummy loop indices
      INTEGER ::   ii, ij                    ! 2D addresses
      REAL(wp) ::  corr ! Flather correction
      REAL(wp) :: zwgt                       ! boundary weight
      REAL(wp) :: elapsed

      !!------------------------------------------------------------------------------
      !!  OPA 9.0, LODYC-IPSL (2003)
      !!------------------------------------------------------------------------------

      ! ---------------------------------!
      ! Flather boundary conditions     :!
      ! ---------------------------------! 
     
      ! Fill temporary array with ssh data (here spgu):
      jgrd = 1
      DO jb = 1, nblenrim(jgrd)
        ji = nbi(jb,jgrd)
        jj = nbj(jb,jgrd)
        spgu(ji, jj) = sshbdy(jb)
#if defined key_bdy_tides
        spgu(ji, jj) = spgu(ji, jj) + sshtide(jb)
#endif
      END DO

      jgrd = 2  !: Flather bc on u-velocity; 
                ! remember that flagu=-1 if normal velocity direction is outward
                ! I think we should rather use after ssh ?

      DO jb = 1, nblenrim(jgrd)
        ji  = nbi(jb,jgrd)
        jj  = nbj(jb,jgrd) 
        jim1 = ji+MAX(0, INT(flagu(jb))) ! T pts i-indice inside the boundary
        jip1 = ji-MIN(0, INT(flagu(jb))) ! T pts i-indice outside the boundary 
     
        corr = - flagu(jb) * sqrt (grav / (hu_e(ji, jj) + 1.e-20) )  &
                                      * ( sshn_e(jim1, jj) - spgu(jip1,jj) )
        ua_e(ji, jj) = ( ubtbdy(jb) + utide(jb) ) * hu_e(ji,jj)
        if ( ln_bdy_fla ) then
          ua_e(ji,jj) = ua_e(ji,jj) + corr * umask(ji,jj,1) * hu_e(ji,jj)
        endif

      END DO
       
      jgrd = 3  !: Flather bc on v-velocity
                ! remember that flagv=-1 if normal velocity direction is outward
                 
      DO jb = 1, nblenrim(jgrd)
        ji  = nbi(jb,jgrd)
        jj  = nbj(jb,jgrd) 
        jjm1 = jj+MAX(0, INT(flagv(jb))) ! T pts j-indice inside the boundary
        jjp1 = jj-MIN(0, INT(flagv(jb))) ! T pts j-indice outside the boundary 
     
        corr = - flagv(jb) * sqrt (grav / (hv_e(ji, jj) + 1.e-20) )  &
                                      * ( sshn_e(ji, jjm1) - spgu(ji,jjp1) )
        va_e(ji, jj) = ( vbtbdy(jb) + vtide(jb) ) * hv_e(ji,jj)
        if ( ln_bdy_fla ) then
          va_e(ji,jj) = va_e(ji,jj) + corr * vmask(ji,jj,1) * hv_e(ji,jj)
        endif

      END DO

      CALL lbc_lnk( ua_e, 'U', 1. ) ! Boundary points should be updated
      CALL lbc_lnk( va_e, 'V', 1. ) !

   END SUBROUTINE bdy_dyn_fla
#endif

#else
   !!=================================================================================
   !!                       ***  MODULE  bdydyn  ***
   !! Ocean dynamics:   Flow relaxation scheme of velocities on unstruc. open boundary
   !!=================================================================================
CONTAINS

   SUBROUTINE bdy_dyn_frs
                              ! No Unstructured open boundaries ==> empty routine
   END SUBROUTINE bdy_dyn_frs

   SUBROUTINE bdy_dyn_fla
                              ! No Unstructured open boundaries ==> empty routine
   END SUBROUTINE bdy_dyn_fla

#endif

END MODULE bdydyn
