MODULE bdyvol
   !!=================================================================================
   !!                       ***  MODULE  bdyvol  ***
   !! Ocean dynamic :  Volume constraint when unstructured boundary 
   !!                  and Free surface are used
   !!=================================================================================
#if   defined key_bdy   &&   defined key_dynspg_flt
   !!---------------------------------------------------------------------------------
   !!   'key_bdy'               and              unstructured open boundary conditions
   !!   'key_dynspg_flt'                                  constant volume free surface
   !!---------------------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers 
   USE dom_oce         ! ocean space and time domain 
   USE phycst          ! physical constants
   USE bdy_oce         ! ocean open boundary conditions
   USE lib_mpp         ! for mppsum
   USE in_out_manager  ! I/O manager
   USE sbc_oce         ! ocean surface boundary conditions

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC bdy_vol        ! routine called by dynspg_flt.h90

   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!---------------------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL  (2003)
   !!---------------------------------------------------------------------------------

CONTAINS

   SUBROUTINE bdy_vol ( kt )
      !!------------------------------------------------------------------------------
      !!                      ***  ROUTINE bdyvol  ***
      !!
      !! ** Purpose : 
      !!      This routine is called in dynspg_flt to control 
      !!      the volume of the system. A correction velocity is calculated
      !!      to correct the total transport through the unstructured OBC. 
      !!      The total depth used is constant (H0) to be consistent with the 
      !!      linear free surface coded in OPA 8.2
      !!
      !! ** Method :  
      !!      The correction velocity (zubtpecor here) is defined calculating
      !!      the total transport through all open boundaries (trans_bdy) minus
      !!      the cumulate E-P flux (zCflxemp) divided by the total lateral 
      !!      surface (bdysurftot) of the unstructured boundary. 
      !!
      !!      zubtpecor = [trans_bdy - zCflxemp ]*(1./bdysurftot)
      !!
      !!      with zCflxemp => sum of (Evaporation minus Precipitation)
      !!                       over all the domain in m3/s at each time step.
      !!
      !!      zCflxemp < 0 when precipitation dominate
      !!      zCflxemp > 0 when evaporation dominate
      !!
      !!      There are 2 options (user's desiderata): 
      !!
      !!         1/ The volume changes according to E-P, this is the default
      !!            option. In this case the cumulate E-P flux are setting to
      !!            zero (zCflxemp=0) to calculate the correction velocity. So
      !!            it will only balance the flux through open boundaries.
      !!            (set volbdy to 0 in tne namelist for this option)
      !!
      !!         2/ The volume is constant even with E-P flux. In this case
      !!            the correction velocity must balance both the flux 
      !!            through open boundaries and the ones through the free
      !!            surface. 
      !!            (set volbdy to 1 in tne namelist for this option)
      !!
      !! History :
      !!   8.5  !  05-01 (J. Chanut, A. Sellar) Original code
      !!        !  06-01 (J. Chanut) Bug correction
      !!----------------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt   ! ocean time-step index

      !! * Local declarations
      INTEGER ::   ji,jj,jb, jgrd, jk
      REAL(wp) ::   zubtpecor
      REAL(wp) ::   zCflxemp
      REAL(wp) ::   ztranst
      !!-----------------------------------------------------------------------------

      IF( kt == nit000 ) THEN 
         IF(lwp) WRITE(numout,*)'        '
         IF(lwp) WRITE(numout,*)'bdy_vol : Correction of velocities along unstructured OBC'
         IF(lwp) WRITE(numout,*)'~~~~~~~'
         IF(lwp) WRITE(numout,*)'        '
      END IF 

      ! 1. Calculate the cumulate surface Flux zCflxemp (m3/s) over all the domain.
      ! ---------------------------------------------------------------------------
 
      zCflxemp = 0.e0

      DO jj = 1, jpj
         DO ji = 1, jpi
            zCflxemp = zCflxemp + ( (emp(ji,jj)*bdytmask(ji,jj)*tmask_i(ji,jj) )/rauw) &
                                  *e1v(ji,jj)*e2u(ji,jj)
         END DO
      END DO
      IF( lk_mpp )   CALL mpp_sum( zCflxemp )   ! sum over the global domain

      ! 2. Barotropic velocity through the unstructured open boundary
      ! -------------------------------------------------------------

      zubtpecor = 0.e0

      jgrd = 2 ! cumulate u component contribution first 
      DO jb = 1, nblenrim(jgrd)
        DO jk = 1, jpkm1
          zubtpecor = zubtpecor + flagu(jb) * ua (nbi(jb,jgrd), nbj(jb,jgrd), jk) &
                                            * e2u(nbi(jb,jgrd), nbj(jb,jgrd)) &
                                            * fse3u(nbi(jb,jgrd), nbj(jb,jgrd), jk)
        END DO
      END DO

      jgrd = 3 ! then add v component contribution
       DO jb = 1, nblenrim(jgrd)
        DO jk = 1, jpkm1
          zubtpecor = zubtpecor + flagv(jb) * va (nbi(jb,jgrd), nbj(jb,jgrd), jk) &
                                            * e1v(nbi(jb,jgrd), nbj(jb,jgrd)) &
                                            * fse3v(nbi(jb,jgrd), nbj(jb,jgrd), jk) 
        END DO
      END DO

      IF( lk_mpp )   CALL mpp_sum( zubtpecor )   ! sum over the global domain


      ! 3. The normal velocity correction
      ! ---------------------------------

      IF (volbdy==1) THEN
        zubtpecor = (zubtpecor - zCflxemp)*(1./bdysurftot) 
      ELSE
        zubtpecor =  zubtpecor*(1./bdysurftot)
      END IF

      IF( lwp .AND. MOD( kt, nwrite ) == 0) THEN
         IF(lwp) WRITE(numout,*)'        '
         IF(lwp) WRITE(numout,*)'bdy_vol : time step :', kt
         IF(lwp) WRITE(numout,*)'~~~~~~~ '
         IF(lwp) WRITE(numout,*)'        '
         IF(lwp) WRITE(numout,*)'          cumulate flux EMP :', zCflxemp,' (m3/s)'
         IF(lwp) WRITE(numout,*)'          total lateral surface of OBC :',bdysurftot,'(m2)'
         IF(lwp) WRITE(numout,*)'          correction velocity zubtpecor :',zubtpecor,'(m/s)'
         IF(lwp) WRITE(numout,*)'        '
      END IF 

      ! 4. Correction of the total velocity on the unstructured 
      !    boundary to respect the mass flux conservation
      ! -------------------------------------------------------

      ztranst = 0.e0

      jgrd = 2 ! correct u component
      DO jb = 1, nblenrim(jgrd)
        DO jk = 1, jpkm1
          ua(nbi(jb, jgrd), nbj(jb, jgrd), jk) = ua(nbi(jb, jgrd), nbj(jb, jgrd), jk) &
                            -flagu(jb) * zubtpecor * umask(nbi(jb, jgrd), nbj(jb, jgrd), jk)
          ztranst = ztranst + flagu(jb) * ua (nbi(jb,jgrd), nbj(jb,jgrd), jk) &
                                        * e2u(nbi(jb,jgrd), nbj(jb,jgrd)) &
                                        * fse3u(nbi(jb,jgrd), nbj(jb,jgrd), jk)
        END DO
      END DO

      jgrd = 3 ! correct v component
       DO jb = 1, nblenrim(jgrd)
        DO jk = 1, jpkm1
          va(nbi(jb, jgrd), nbj(jb, jgrd), jk) = va(nbi(jb, jgrd), nbj(jb, jgrd), jk) &
                            -flagv(jb) * zubtpecor * vmask(nbi(jb, jgrd), nbj(jb, jgrd), jk)
          ztranst = ztranst + flagv(jb) * va (nbi(jb,jgrd), nbj(jb,jgrd), jk) &
                                        * e1v(nbi(jb,jgrd), nbj(jb,jgrd)) &
                                        * fse3v(nbi(jb,jgrd), nbj(jb,jgrd), jk)
        END DO
      END DO
      
      IF( lk_mpp )   CALL mpp_sum( ztranst )   ! sum over the global domain
 
      ! 5. Check the cumulate transport through unstructured OBC
      !    once barotropic velocities corrected
      ! --------------------------------------------------------

      IF( lwp .AND. MOD( kt, nwrite ) == 0) THEN
         IF(lwp) WRITE(numout,*)'        '
         IF(lwp) WRITE(numout,*)'          Cumulate transport ztranst =', ztranst,'(m3/s)'
         IF(lwp) WRITE(numout,*)'        '
      END IF 

   END SUBROUTINE bdy_vol

#else
   !!---------------------------------------------------------------------------------
   !!  Default option :                                                   Empty module
   !!---------------------------------------------------------------------------------
CONTAINS
   SUBROUTINE bdy_vol        ! Empty routine
   END SUBROUTINE bdy_vol
#endif

   !!=================================================================================
END MODULE bdyvol
